import pygame
from pygame.sprite import Sprite
from pygame.rect import Rect
import Sounds, Weapons
from random import randint, choice
from copy import copy
import Graphics_Config

print "Loading Objects..."


class Shrapnel(Sprite):
	# Shrapnel is generated by a vareity of impacts and
	# explosions.
	def __init__(self, location, direction, images, Level):
		Sprite.__init__(self)
		self.Level = Level
		self.rect = Rect(*tuple(location)+(8,8))
		self.images = images
		self.speed = [direction[0]+randint(-2,2),
			direction[1]+randint(-2,2)
			]
		self.image = choice(images)
	def update(self):
		if not randint(0,20):
			self.kill()
		self.rect.move_ip(self.speed)
		self.image = choice(self.images)
		
	
class Explosion(Sprite):
	# Explosions expode. Duh.
	def __init__(self, location, images, Level):
		self.Level = Level
		self.images = images
		self.image = images[0]
		self.rect = Rect(*tuple(location)+self.image.get_size())
		self.frame = -1
		Sprite.__init__(self)
	def update(self):
		if self.frame == len(self.images): self.kill(); return
		self.image = self.images[self.frame]
		self.frame += 1


class Block(Sprite):
	name = 'Block'
	images = Graphics_Config.Blocks
	cln = 0
	amg = False
	speed = [0,0]
	shield = 80
	#
	# Blocks are destructable objects that are lethal to players and
	# projectiles. When hit by either, however, they lose hit points.
	# When the block's hit points are expended, it perishes.
	# Blocks die in a shower of debris.
	def __init__(self, x, y, type, Level):
		self.Level = Level
		if isinstance(type, tuple):
			self.attr = type[1]
			type = 0
		try:
			self.image = self.images[type]
		except:
			self.image = self.images[0]
		self.rect = Rect(*((x, y)+self.image.get_size()))
		self.type = type
		# 27 July, 2004, 5:14 AM: First run with graphics. Yay.
		Sprite.__init__(self)
	def destroy(self, type=0):
		Sounds.BExplode.play()
		i = 0
		if self.Level.mineflag < 4:
			m = randint(10,15)
			while i < m:
				self.Level.effects.add(Shrapnel(
					[self.rect.x,self.rect.y], 
					[randint(-10,10), randint(-10,10)], 
					Graphics_Config.Shrapnel_Grey, self.Level
					))
				i += 1
	
			i = 0
			m = randint(15,20)
			while i < m:
				self.Level.effects.add(Shrapnel(
					[self.rect.x,self.rect.y], 
					[randint(-6,6), randint(-6,6)], 
					Graphics_Config.Shrapnel_Purple, self.Level
					))
				i += 1
		self.Level.effects.add(Explosion(
			[self.rect.x-16, self.rect.y-16],
			Graphics_Config.Explosion, self.Level
			))
		self.kill() # Needs special effects.
	def collide(self, player):
		if not self.cln: 
			#self.cln =  1
			player.speed = [player.speed[1]*-1, player.speed[0]]

			Sounds.Hit.play()
			player.collided_block = True
			player.rect.move_ip(player.speed)
			if ( player.rect.x+player.rect.w/2 > self.rect.x+4 and player.rect.x+player.rect.w/2 < self.rect.x+self.rect.w-4 ) and (
				player.rect.y+player.rect.h/2 > self.rect.y+4 and player.rect.y+player.rect.h/2 < self.rect.y+self.rect.h-4 ):
				self.shield -= 1000
				player.shield -= 1000
				player.lastdamage = 'being crushed into a wall'
				player.lasthitter = self
class Switch(Block):
	name = 'Switch'
	images = Graphics_Config.Switch
	shield = 1
	state = True
	def destroy(self, type=0):
		self.state = not self.state
		if self.scripts[self.state]:
			self.scripts[self.state](self)
		self.shield = 1
		self.image = self.images[self.state]
	def __init__(self, x, y, onscript, offscript, Level):
		self.Level = Level
		self.scripts = [offscript, onscript]
		self.image = self.images[0]
		self.state = False
		self.rect = Rect(*((x, y)+self.image.get_size()))
		Sprite.__init__(self)

	
class Wall(Block):
	name = 'Wall'
	images = Graphics_Config.Walls
	# A Wall is an immortal block. :)
	def destroy(self):
		pass

class BArea(Block):
	shield = 300
	n = 0
	images = Graphics_Config.BArea
	def collide(self, player):
		pass
	def update(self):
		self.n += 1
		if self.n > self.type:
			self.kill()
			self.Level.blocks.add(Block(
				self.rect.x, self.rect.y,
				0, self.Level
				))
			self.Level.effects.add(Explosion(
				[self.rect.x, self.rect.y],
				Graphics_Config.PortFlash, self.Level
				))
			Sounds.TimeBlock.play()



class TBlock(Block):
	n = 0
	shield = 7000
	images = Graphics_Config.TBlock
	def update(self):
		self.n += 1
		if self.n > self.type:
			self.kill()
			self.Level.blocks.add(Block(
				self.rect.x, self.rect.y,
				0, self.Level
				))
			self.Level.effects.add(Explosion(
				[self.rect.x, self.rect.y],
				Graphics_Config.PortFlash, self.Level
				))
			Sounds.TimeBlock.play()




class MobileBlock(Block):
	shield = 1000
	bouncey = True
	dead = False
	lasthitter  = None
	converted = False
	id = -1
	secondary = None
	name = 'a Rampant Block'
	maxspeed = 0
	count = 0
	oshield = 1000
	images = Graphics_Config.MobileBlock
	def collide(self, player):
		player.collided_block = True
		if not self.converted:
			self.speed = [player.speed[0]/2, player.speed[1]/2]
			player.speed = [player.speed[0]/-2, player.speed[1]/-2]
			player.rect.move_ip(player.speed)
			player.rect.move_ip(player.speed)
			self.Level.blocks.remove(self)
			self.Level.players.add(self)
			self.converted = True

	def update(self):
		#if self.rect.x > 992 or self.rect.x < 0 or self.rect.y > 736 or self.rect.y < 0: self.speed = [0,0]
		self.rect.move_ip(self.speed)
		if self.shield < 0: self.destroy(); self.dead = True
		if self.shield < self.oshield:
			self.count += 1
			self.speed = copy(self.lasthitter.speed)
			self.oshield = self.shield
			if not self.converted:
				self.Level.blocks.remove(self)
				self.Level.players.add(self)
				self.converted = True
			if self.count > 10:
				self.shield -= 1000
				try:
					self.lasthitter.shield -= 1000
					self.lasthitter.lastdamage = 'a moving block'
				except:
					pass
				self.lastdamage = 'a collision'
		else:
			if self.count: self.count -= 0
			
class GBlock(Block):
	object = Block
	images = Graphics_Config.GBlock
	fn = 0
	param = 0
	shield = 80
	amg = True
	def destroy(self, type=0):
		Sounds.BExplode.play()
		self.Level.effects.add(Explosion(
			[self.rect.x-16, self.rect.y-16],
			Graphics_Config.PortFlash, self.Level
			))
		x = self.rect.x-32
		while x <= self.rect.x+32:
			y = self.rect.y-32
			while y <= self.rect.y+32:
				self.Level.blocks.add(choice(self.object)(
					x, y,
					(self.type, self.param), self.Level
					))
				y += 32
			x += 32
			self.Level.bounds.add(Bound(self.rect.x-32, self.rect.y-32,self.rect.x+32,self.rect.y+32))
		self.kill()
	def collide(self, player):
		if not self.cln: 
			self.cln =  1
			player.speed = [player.speed[1]*-1, player.speed[0]*-1]
			Sounds.Hit.play()
			player.collided_block = True
			player.rect.move_ip(player.speed)
			if ( player.rect.x+player.rect.w/2 > self.rect.x-8 and player.rect.x+player.rect.w/2 < self.rect.x+self.rect.w+8 ) and (
				player.rect.y+player.rect.h/2 > self.rect.y-8 and player.rect.y+player.rect.h/2 < self.rect.y+self.rect.h+8 ):
				self.shield -= 1000
				player.shield -= 1000
				player.lastdamage = 'being crushed into a wall'
				player.lasthitter = self
	def update(self):
		try:
			self.image = self.images[8-self.shield/10]
		except:
			self.shield = 2
	def __init__(self, x, y, type, Level):
		self.type = type[0]
		self.Level = Level
		self.object = [
			[Block],
			[Wall],
			[Block,Wall],
			[GBlock,Block,Block,Wall],
			[GBlock,Block,Block]
			][self.type]
		self.image = self.images[0]
		self.param = type[1]
		self.rect = Rect(*((x, y)+(32,32)))
		Sprite.__init__(self)


class TargetDot(Sprite):
	image = Graphics_Config.TargetDot[0]
	def __init__(self, x,y,speed, player):
		self.player=player
		self.rect = Rect(x,y,4,4)
		Sprite.__init__(self)
		self.n = 0
		self.speed = speed
		if self.player.secondary and not self.player.recycle2timer:
			self.image = Graphics_Config.TargetDot[0]
		else:
			self.image = Graphics_Config.TargetDot[1]
	def update(self):
		if self.n: self.kill()
		self.rect.move_ip(self.speed)
		self.n += 1

class Item(Sprite):
	# Parent Class of various items with distinguished only
	# By image and their collision effect.
	# Actual items must define a collide() method and an init() method.
	# (Which must set the graphic.)
	nil = None
	name = "An unknown item"
	sound = Sounds.Bonus
	nhitme = False
	def __init__(self, x, y, type, Level):
		self.Level = Level
		self.rect = Rect(x,y, 24,24)
		self.init(type)
		Sprite.__init__(self)
	def destroy(self, type=0):
		self.kill()
		if not type: self.sound.play()
	
		

class ShieldItem(Item):
	# Shield Restore Item.
	image = Graphics_Config.Items[0]
	def init(self, value):
		self.value = abs(value)
		self.name = '+%d shield bonus item'%self.value
	def collide(self, player):
		player.shield += self.value
		self.Level.prnt = "%s got a +%d shield bonus! (%d shields total)"%(player.name,
			self.value, player.shield)
		self.destroy()
class GorgonItem(Item):
	image = Graphics_Config.GorgonItem
	def collide(self, player):
		try:
			player.allow_gorgon = True
			self.destroy()
		except:
			pass
	def init(self, value):
		self.name = 'Stone of Gorgon'

class SpeedItem(Item):
	# Speed Doubler Item.
	name = 'speed bonus item'
	image = Graphics_Config.Items[1]
	def init(self, type):
		pass
	def collide(self, player):
		self.Level.prnt = "%s got a speed bonus!"%(player.name)
		player.maxspeed += 1
		self.destroy()

class Teleporter(Item):	
	image = Graphics_Config.Tele[0]
	def init(self,value):
		self.tx, self.ty = value
		self.image = choice(Graphics_Config.Tele)
		self.name = 'teleporter to (%d,%d)'%(self.tx,self.ty)
	def collide(self,player):
		#player.speed = [0,0]
		Sounds.Tele.play(1, 100)
		self.Level.effects.add(Explosion(
				[self.rect.x-8,self.rect.y-8], Graphics_Config.PortFlash,
				self.Level
			))
		self.Level.effects.add(Explosion(
				[self.tx+4,self.ty+8], Graphics_Config.PortFlash,
				self.Level
			))
		player.rect.x,player.rect.y = self.tx+4, self.ty+4
	def update(self):
		self.image = choice(Graphics_Config.Tele)


# Secondary-Weapon-Granting Item.
class WeaponItem(Item):
	def collide(self, player):
		if player.secondary == Weapons.WList[self.type]:
			player.recycle2 /= 2
			self.Level.prnt = "%s got more %s weapons."%(
				player.name, player.secondary.name
				)
			player.scount += Weapons.WList[self.type].scount
		else:
			player.secondary = Weapons.WList[self.type]
			player.recycle2 = player.secondary.recycle
			player.scount = player.secondary.scount
			self.Level.prnt = "%s is now armed with %s weapons!"%(
				player.name, player.secondary.name
				)
		self.destroy()
		
	def init(self, type):
		self.type = type
		self.name = '%s granter item'%Weapons.WList[self.type].name
		self.image = Graphics_Config.Items[self.type+1]

class Bound(Sprite):
	# Boundry class, usualy seen keeping players from
	# Hitting blocks and walls.
	def __init__(self, x1, y1, x2, y2):
		self.image = Graphics_Config.NilImg
		self.rect = Rect(x1,y1,x2-x1, y2-y1)
		Sprite.__init__(self)
Solids = [Block, Wall, BArea, TBlock, GBlock]
